var fs = require('fs');
var path = require("path");
var consoleFileLogger = require("../sl-logger.js").consoleFileLogger;
var sourceMap = require("source-map");
var globalErrorHandler = require('../global-error-handler');

function readSourceMaps(fullFilename, logger) {
    if (!fullFilename) return;
    var sourceMapsFilename = fullFilename + '.map';
    var contents, rawSourceMapJsonData, consumer;
    var log = logger ? logger.child({
        methodName: "readSourceMaps"
    }) : consoleFileLogger("readSourceMaps");
    var hasSourceMaps = fs.existsSync(sourceMapsFilename);
    if(!hasSourceMaps){
        return null;
    }
    try {
        contents = fs.readFileSync(sourceMapsFilename).toString();
    }catch (e) {
        log.warn("failed to read sourcemap file content. " +e);
        return null;
    }

    try {
        rawSourceMapJsonData = JSON.parse(contents);
    } catch (e) {
        globalErrorHandler.setLastError(e);
        return null;
    }

    try {
        log.debug('Processing source maps from file: ' + sourceMapsFilename);
        consumer = new sourceMap.SourceMapConsumer(rawSourceMapJsonData);
        tryOverridePathInSourceMap(consumer, fullFilename, logger)
                
        return consumer;
    } catch (e) {
        globalErrorHandler.setLastError(e);
        return null;
    }
}

/**
 * By default, source maps store relative path to the file.
 * This methods try to convert the path to an absolute path for easier processing
 * later in the build process.
 */
function tryOverridePathInSourceMap(consumer, fullFilename, logger){
    if (!consumer || !consumer.sources || consumer.sources.length == 0){
        return;
    }

    var log = logger ? logger.child({
        methodName: "tryOverridePathInSourceMap"
    }) : consoleFileLogger("tryOverridePathInSourceMap");

    var parsedPath = path.parse(fullFilename);
    var newSources = [];

    for (var i=0; i < consumer.sources.length; i++){
        var sourceFile =  consumer["_sources"]["_array"][i];
        var absolutePath = "";
        if (path.isAbsolute(sourceFile)){
            absolutePath = sourceFile
        }else {
            absolutePath = parsedPath.dir + "/" +sourceFile;
            absolutePath = path.normalize(absolutePath);
            absolutePath = path.resolve(absolutePath);
        }

        log.debug("Path '" + consumer.sources[i] + "' is overridden to '" + absolutePath+ "'.")
        newSources.push(absolutePath);
    }
    
    consumer["_sources"]["_array"] = newSources;

    if (consumer.sources.length > 1){
        log.warn("File '" + fullFilename+ "' has multiple sources. An assumption is made that they all originate can be found in folder '" + parsedPath.dir + "'.");
    }
}

function ActualFile(filename, fullFilename) {
    this.getFilename = function ActualFile$getFilename() {
        return filename;
    }
    this.getFullFilename = function ActualFile$getFullFilename() {
        return fullFilename;
    }
    this.getContent = function ActualFile$getContent() {
        var content = fs.readFileSync(fullFilename, 'utf-8');
        return content;
    }
    this.readSourceMaps = function ActualFile$readSourceMaps(logger) {
        return readSourceMaps(fullFilename, logger);
    }
}
module.exports = ActualFile;